<?php

namespace FiloBlu\AdvancedCustomOptions\Model\ResourceModel\Product\Option\Value;

use Magento\Store\Model\Store;

/**
 * Class Collection
 * @package FiloBlu\AdvancedCustomOptions\Model\ResourceModel\Product\Option\Value
 */
class Collection extends \Magento\Catalog\Model\ResourceModel\Product\Option\Value\Collection
{

    /**
     * Add image to result
     *
     * @param int $storeId
     * @return $this
     */
    public function getValues($storeId)
    {
        parent::getValues($storeId);

        $this->addImageToResult($storeId);
        $this->addSalableToResult($storeId);
        $this->addDefaultToResult($storeId);

        return $this;
    }

    /**
     * Add Image to Result
     *
     * @param $storeId
     * @return $this
     */
    public function addImageToResult($storeId)
    {
        $optionImageTable = $this->getTable('catalog_product_option_type_image');
        $imageExpr = $this->getConnection()->getCheckSql(
            'store_value_image.option_image IS NULL',
            'default_value_image.option_image',
            'store_value_image.option_image'
        );

        $joinExpr = 'store_value_image.option_type_id = main_table.option_type_id AND ' .
            $this->getConnection()->quoteInto('store_value_image.store_id = ?', $storeId);
        $this->getSelect()->join(
            ['default_value_image' => $optionImageTable],
            'default_value_image.option_type_id = main_table.option_type_id',
            ['default_image' => 'option_image']
        )->joinLeft(
            ['store_value_image' => $optionImageTable],
            $joinExpr,
            ['store_image' => 'option_image', 'option_image' => $imageExpr]
        )->where(
            'default_value_image.store_id = ?',
            Store::DEFAULT_STORE_ID
        );

        return $this;
    }


    /**
     * Add Saleable to result
     *
     * @param $storeId
     * @return $this
     */
    public function addSalableToResult($storeId)
    {
        $optionConfigurationTable = $this->getTable('catalog_product_option_type_configuration');
        $configurationExpr = $this->getConnection()->getCheckSql(
            'store_value_salable.option_salable IS NULL',
            'default_value_salable.option_salable',
            'store_value_salable.option_salable'
        );

        $joinExpr = 'store_value_salable.option_type_id = main_table.option_type_id AND ' .
            $this->getConnection()->quoteInto('store_value_salable.store_id = ?', $storeId);
        $this->getSelect()->join(
            ['default_value_salable' => $optionConfigurationTable],
            'default_value_salable.option_type_id = main_table.option_type_id',
            ['default_value_salable' => 'option_salable']
        )->joinLeft(
            ['store_value_salable' => $optionConfigurationTable],
            $joinExpr,
            ['store_salable' => 'option_salable', 'option_salable' => $configurationExpr]
        )->where(
            'default_value_salable.store_id = ?',
            Store::DEFAULT_STORE_ID
        );

        return $this;
    }

    /**
     * Add default to result
     *
     * @param $storeId
     * @return $this
     */
    public function addDefaultToResult($storeId)
    {
        $optionConfigurationTable = $this->getTable('catalog_product_option_type_configuration');
        $configurationExpr = $this->getConnection()->getCheckSql(
            'store_value_default.option_default IS NULL',
            'default_value_default.option_default',
            'store_value_default.option_default'
        );

        $joinExpr = 'store_value_default.option_type_id = main_table.option_type_id AND ' .
            $this->getConnection()->quoteInto('store_value_default.store_id = ?', $storeId);
        $this->getSelect()->join(
            ['default_value_default' => $optionConfigurationTable],
            'default_value_default.option_type_id = main_table.option_type_id',
            ['default_value_default' => 'option_default']
        )->joinLeft(
            ['store_value_default' => $optionConfigurationTable],
            $joinExpr,
            ['store_default' => 'option_default', 'option_default' => $configurationExpr]
        )->where(
            'default_value_default.store_id = ?',
            Store::DEFAULT_STORE_ID
        );

        return $this;
    }
}
