<?php
/**
 * Copyright © 2019 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\AdvancedCustomOptions\Setup;

use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * @codeCoverageIgnore
 */
class InstallSchema implements InstallSchemaInterface
{
    /**
     * OPTION IMAGE TABLE
     */
    const OPTION_IMAGE_TABLE = 'catalog_product_option_type_image';

    /**
     * OPTION TITLE TABLE
     */
    const OPTION_TITLE_TABLE = 'catalog_product_option_type_value';

    /**
     * OPTION CONFIGURATION TABLE
     */
    const OPTION_CONFIGURATION_TABLE = 'catalog_product_option_type_configuration';

    /**
     * PRODUCT OPTION TABLE
     */
    const PRODUCT_OPTION_TABLE = 'catalog_product_option';

    /**
     * STORE TABLE
     */
    const STORE_TABLE = 'store';

    /**
     * OPTION TYPE ID FIELD
     */
    const OPTION_TYPE_ID_FIELD = 'option_type_id';

    /**
     * OPTION ID FIELD
     */
    const OPTION_ID_FIELD = 'option_id';

    /**
     * STORE ID FIELD
     */
    const STORE_ID_FIELD = 'store_id';

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        //Setup image option table
        $optionImageTable = $this->createOptionImageTable($setup, $context);
        $setup->getConnection()->createTable($optionImageTable);

        //Setup configurable option table
        $optionConfigurationTable = $this->createOptionConfigurationTable($setup, $context);
        $setup->getConnection()->createTable($optionConfigurationTable);

        $setup->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return Table
     * @throws \Zend_Db_Exception
     */
    public function createOptionImageTable(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        return $setup->getConnection()->newTable(
            $setup->getTable(self::OPTION_IMAGE_TABLE)
        )->addColumn(
            'option_type_image_id',
            Table::TYPE_INTEGER,
            10,
            ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
            'Option Type Image ID'
        )->addColumn(
            self::OPTION_TYPE_ID_FIELD,
            Table::TYPE_INTEGER,
            10,
            ['unsigned' => true, 'nullable' => false, 'identity' => false, 'default' => 0],
            'Option Type ID'
        )->addColumn(
            self::STORE_ID_FIELD,
            Table::TYPE_SMALLINT,
            5,
            ['unsigned' => true, 'nullable' => false, 'identity' => false, 'default' => 0],
            'Store ID'
        )->addColumn(
            'option_image',
            Table::TYPE_TEXT,
            255,
            [],
            'Option image'
        )->addIndex(
            $setup->getIdxName(
                self::OPTION_IMAGE_TABLE,
                [self::OPTION_TYPE_ID_FIELD,self::STORE_ID_FIELD],
                AdapterInterface::INDEX_TYPE_UNIQUE
            ),
            [self::OPTION_TYPE_ID_FIELD,self::STORE_ID_FIELD],
            ['type' => AdapterInterface::INDEX_TYPE_UNIQUE]
        )->addIndex(
            $setup->getIdxName(
                self::OPTION_IMAGE_TABLE,
                [self::STORE_ID_FIELD],
                AdapterInterface::INDEX_TYPE_INDEX
            ),
            [self::STORE_ID_FIELD],
            ['type' => AdapterInterface::INDEX_TYPE_INDEX]
        )->addForeignKey(
            $setup->getFkName(
                self::OPTION_IMAGE_TABLE,
                self::OPTION_TYPE_ID_FIELD,
                self::OPTION_TITLE_TABLE,
                self::OPTION_TYPE_ID_FIELD
            ),
            self::OPTION_TYPE_ID_FIELD,
            $setup->getTable(self::OPTION_TITLE_TABLE),
            self::OPTION_TYPE_ID_FIELD,
            Table::ACTION_CASCADE
        )->addForeignKey(
            $setup->getFkName(
                self::OPTION_IMAGE_TABLE,
                self::STORE_ID_FIELD,
                self::STORE_TABLE,
                self::STORE_ID_FIELD
            ),
            self::STORE_ID_FIELD,
            $setup->getTable(self::STORE_TABLE),
            self::STORE_ID_FIELD,
            Table::ACTION_CASCADE
        )->setComment(
            'Catalog Product Option Type Image'
        );
    }

    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return Table
     * @throws \Zend_Db_Exception
     */
    public function createOptionConfigurationTable(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        return $setup->getConnection()->newTable(
            $setup->getTable(self::OPTION_CONFIGURATION_TABLE)
        )->addColumn(
            'option_type_configuration_id',
            Table::TYPE_INTEGER,
            10,
            ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
            'Option Type Configuration ID'
        )->addColumn(
            self::OPTION_TYPE_ID_FIELD,
            Table::TYPE_INTEGER,
            10,
            ['unsigned' => true, 'nullable' => false, 'identity' => false, 'default' => 0],
            'Option Type ID'
        )->addColumn(
            self::STORE_ID_FIELD,
            Table::TYPE_SMALLINT,
            5,
            ['unsigned' => true, 'nullable' => false, 'identity' => false, 'default' => 0],
            'Store ID'
        )->addColumn(
            'option_salable',
            Table::TYPE_SMALLINT,
            5,
            ['default' => 1],
            'Option salable'
        )->addColumn(
            'option_default',
            Table::TYPE_SMALLINT,
            5,
            ['default' => 0],
            'Option default'
        )->addIndex(
            $setup->getIdxName(
                self::OPTION_CONFIGURATION_TABLE,
                [self::OPTION_TYPE_ID_FIELD,self::STORE_ID_FIELD],
                AdapterInterface::INDEX_TYPE_UNIQUE
            ),
            [self::OPTION_TYPE_ID_FIELD,self::STORE_ID_FIELD],
            ['type' => AdapterInterface::INDEX_TYPE_UNIQUE]
        )->addIndex(
            $setup->getIdxName(
                self::OPTION_CONFIGURATION_TABLE,
                [self::STORE_ID_FIELD],
                AdapterInterface::INDEX_TYPE_INDEX
            ),
            [self::STORE_ID_FIELD],
            ['type' => AdapterInterface::INDEX_TYPE_INDEX]
        )->addForeignKey(
            $setup->getFkName(
                self::OPTION_CONFIGURATION_TABLE,
                self::OPTION_TYPE_ID_FIELD,
                self::OPTION_TITLE_TABLE,
                self::OPTION_TYPE_ID_FIELD
            ),
            self::OPTION_TYPE_ID_FIELD,
            $setup->getTable(self::OPTION_TITLE_TABLE),
            self::OPTION_TYPE_ID_FIELD,
            Table::ACTION_CASCADE
        )->addForeignKey(
            $setup->getFkName(
                self::OPTION_CONFIGURATION_TABLE,
                self::STORE_ID_FIELD,
                self::STORE_TABLE,
                self::STORE_ID_FIELD
            ),
            self::STORE_ID_FIELD,
            $setup->getTable(self::STORE_TABLE),
            self::STORE_ID_FIELD,
            Table::ACTION_CASCADE
        )->setComment(
            'Catalog Product Option Type Configuration'
        );
    }
}
